@@ -45,7 +45,7 @@
 
 static GHashTable *window_hash = NULL;
 
-/* Keep 0-9 in sync with the numbers in the WindowState enum. Yeah I'm
+/* Keep 0-10 in sync with the numbers in the WindowState enum. Yeah I'm
  * a loser.
  */
 #define COMPRESS_STATE(window)                          \
@@ -58,7 +58,8 @@ static GHashTable *window_hash = NULL;
     ((window)->priv->is_sticky           << 6) |        \
     ((window)->priv->is_hidden           << 7) |        \
     ((window)->priv->is_fullscreen       << 8) |        \
-    ((window)->priv->is_urgent           << 9) )
+    ((window)->priv->is_urgent           << 9) |        \
+    ((window)->priv->demands_attention   << 10) )
 
 struct _NetkWindowPrivate
 {
@@ -105,6 +106,7 @@ struct _NetkWindowPrivate
     guint is_hidden:1;
     guint is_fullscreen:1;
     guint is_urgent : 1;
+    guint demands_attention : 1;
 
     /* _NET_WM_STATE_HIDDEN doesn't map directly into an
      * externally-visible state (it determines the WM_STATE
@@ -130,6 +132,9 @@ struct _NetkWindowPrivate
     guint need_update_transient_for:1;
     guint need_update_wmclass : 1;
     guint need_update_wmhints : 1;
+
+    /* if available */
+    char *client_machine;
 };
 
 enum
@@ -171,6 +176,8 @@ static void unqueue_update (NetkWindow * window);
 static void queue_update (NetkWindow * window);
 static void force_update_now (NetkWindow * window);
 
+static NetkWindow *find_last_transient_for (GList * windows, Window xwindow);
+
 static gpointer parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
 
@@ -193,6 +200,7 @@ netk_window_get_type (void)
             sizeof (NetkWindow),
             0,                  /* n_preallocs */
             (GInstanceInitFunc) netk_window_init,
+            NULL                /* value_table */
         };
 
         object_type =
@@ -210,6 +218,7 @@ netk_window_init (NetkWindow * window)
 
     window->priv->name = g_strdup (FALLBACK_NAME);;
     window->priv->icon_name = NULL;
+    window->priv->client_machine = NULL;
     window->priv->workspace = ALL_WORKSPACES;
 
     window->priv->icon_cache = p_netk_icon_cache_new ();
@@ -283,6 +292,7 @@ netk_window_finalize (GObject * object)
 
     p_netk_icon_cache_free (window->priv->icon_cache);
 
+    g_free (window->priv->client_machine);
     g_free (window->priv->name);
     g_free (window->priv->icon_name);
     g_free (window->priv->session_id);
@@ -363,6 +373,8 @@ p_netk_window_create (Window xwindow, NetkScreen * screen)
     window->priv->session_id = p_netk_get_session_id (window->priv->xwindow);
 
     window->priv->pid = p_netk_get_pid (window->priv->xwindow);
+    
+    window->priv->client_machine = p_netk_get_client_machine (window->priv->xwindow);
 
     p_netk_get_window_geometry (NETK_SCREEN_XSCREEN (window->priv->screen),
                                 xwindow, &window->priv->x, &window->priv->y,
@@ -506,6 +518,20 @@ netk_window_get_session_id_utf8 (NetkWindow * window)
     return window->priv->session_id_utf8;
 }
 
+/**
+ * netk_window_get_pid:
+ * @window: a #NetkWindow
+ *
+ * Gets the client machine FQDN of @window, if available.
+ *
+ * Return value: FQDN of machine of @window, or NULL if none available
+ **/
+const char *
+netk_window_get_client_machine (NetkWindow * window)
+{
+	return window->priv->client_machine;
+}
+
 /**
  * netk_window_get_pid:
  * @window: a #NetkWindow
@@ -637,14 +663,6 @@ netk_window_is_skip_tasklist (NetkWindow * window)
     return window->priv->skip_taskbar;
 }
 
-gboolean
-netk_window_is_fullscreen (NetkWindow * window)
-{
-    g_return_val_if_fail (NETK_IS_WINDOW (window), FALSE);
-
-    return window->priv->is_fullscreen;
-}
-
 void
 netk_window_set_skip_tasklist (NetkWindow * window, gboolean skip)
 {
@@ -654,6 +672,14 @@ netk_window_set_skip_tasklist (NetkWindow * window, gboolean skip)
                          p_netk_atom_get ("_NET_WM_STATE_SKIP_TASKBAR"), 0);
 }
 
+gboolean
+netk_window_is_fullscreen (NetkWindow * window)
+{
+    g_return_val_if_fail (NETK_IS_WINDOW (window), FALSE);
+
+    return window->priv->is_fullscreen;
+}
+
 void
 netk_window_set_fullscreen (NetkWindow * window, gboolean fullscreen)
 {
@@ -664,6 +690,67 @@ netk_window_set_fullscreen (NetkWindow * window, gboolean fullscreen)
                         p_netk_atom_get ("_NET_WM_STATE_FULLSCREEN"), 0);
 }
 
+/**
+ * netk_window_demands_attention:
+ * @window: a #NetkWindow
+ *
+ * If the window demands attention returns %TRUE. This state may change
+ * anytime a state_changed signal gets emitted, as it can depend on flags
+ * such as the demands_attention and is_urgent hints.
+ *
+ * Return value: %TRUE if window demands attention
+ **/
+gboolean
+netk_window_demands_attention (NetkWindow * window)
+{
+    g_return_val_if_fail (NETK_IS_WINDOW (window), FALSE);
+
+    return window->priv->demands_attention || window->priv->is_urgent;
+}
+
+/* Return whether one of the transients of @window demands attention */
+static gboolean
+transient_demands_attention (NetkWindow * window)
+{
+    GList *windows;
+    NetkWindow *transient;
+
+    if (!NETK_IS_WINDOW (window))
+        return FALSE;
+
+    windows = netk_screen_get_windows_stacked (window->priv->screen);
+
+    transient = window;
+    while ((transient =
+            find_last_transient_for (windows, transient->priv->xwindow)))
+    {
+        /* catch transient cycles */
+        if (transient == window)
+            return FALSE;
+
+        if (netk_window_demands_attention (transient))
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
+/**
+ * netk_window_or_transient_demands_attention:
+ * @window: a #NetkWindow
+ *
+ * If the window or one of its transients demands attention returns %TRUE.
+ * This state may change anytime a state_changed signal gets emitted.
+ *
+ * Return value: %TRUE if window or one of its transients demands attention
+ **/
+gboolean
+netk_window_or_transient_demands_attention (NetkWindow * window)
+{
+    return netk_window_demands_attention (window) ||
+        transient_demands_attention (window);
+}
+
 /**
  * netk_window_is_sticky:
  * @window: a #NetkWindow
@@ -1428,6 +1515,7 @@ update_state (NetkWindow * window)
         window->priv->skip_taskbar = FALSE;
         window->priv->skip_pager = FALSE;
         window->priv->net_wm_state_hidden = FALSE;
+        window->priv->demands_attention = FALSE;
 
         atoms = NULL;
         n_atoms = 0;
@@ -1454,6 +1542,8 @@ update_state (NetkWindow * window)
                 window->priv->skip_taskbar = TRUE;
             else if (atoms[i] == p_netk_atom_get ("_NET_WM_STATE_SKIP_PAGER"))
                 window->priv->skip_pager = TRUE;
+            else if (atoms[i] == p_netk_atom_get ("_NET_WM_STATE_DEMANDS_ATTENTION"))
+                window->priv->demands_attention = TRUE;
 
             ++i;
         }
