@@ -37,7 +37,7 @@
 #include "netk-screen.h"
 #include "netk-private.h"
 #include "netk-inlinepixbufs.h"
-#include "xfce-icontheme.h"
+#include "icons.h"
 
 #ifndef XUrgencyHint
 #define XUrgencyHint (1L << 8)
@@ -262,7 +262,7 @@ p_netk_get_text_property (Window xwindow, Atom atom)
         {
             xfce_utf8_remove_controls((gchar *) retval, -1, NULL);
         }
-        if (text.nitems > 0)
+        if (text.value)
             XFree (text.value);
     }
     else
@@ -772,6 +772,7 @@ p_netk_close (Screen * screen, Window xwindow)
     xev.xclient.data.l[0] = 0;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -826,6 +827,7 @@ p_netk_keyboard_size (Screen * screen, Window xwindow)
     xev.xclient.data.l[1] = 0;  /* unused */
     xev.xclient.data.l[2] = _NET_WM_MOVERESIZE_SIZE_KEYBOARD;
     xev.xclient.data.l[3] = 0;  /* unused */
+    
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -851,6 +853,7 @@ p_netk_change_state (Screen * screen, Window xwindow, gboolean add,
     xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
     xev.xclient.data.l[1] = state1;
     xev.xclient.data.l[2] = state2;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -871,6 +874,7 @@ p_netk_change_workspace (Screen * screen, Window xwindow, int new_space)
     xev.xclient.data.l[0] = new_space;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -891,6 +895,7 @@ p_netk_activate (Screen * screen, Window xwindow)
     xev.xclient.data.l[0] = 0;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -911,6 +916,7 @@ p_netk_activate_workspace (Screen * screen, int new_active_space)
     xev.xclient.data.l[0] = new_active_space;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display, RootWindowOfScreen (screen), False,
                 SubstructureRedirectMask | SubstructureNotifyMask, &xev);
@@ -931,6 +937,7 @@ p_netk_change_viewport (Screen * screen, int x, int y)
     xev.xclient.data.l[0] = x;
     xev.xclient.data.l[1] = y;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (gdk_display,
                 RootWindowOfScreen (screen),
@@ -953,6 +960,7 @@ p_netk_toggle_showing_desktop (Screen * screen, gboolean show)
     xev.xclient.data.l[0] = show != FALSE;
     xev.xclient.data.l[1] = 0;
     xev.xclient.data.l[2] = 0;
+    xev.xclient.data.l[3] = 0;
 
     XSendEvent (DisplayOfScreen (screen),
                 RootWindowOfScreen (screen),
@@ -1682,7 +1690,7 @@ p_netk_icon_cache_new (void)
 {
     NetkIconCache *icon_cache;
 
-    icon_cache = g_new (NetkIconCache, 1);
+    icon_cache = g_new0 (NetkIconCache, 1);
 
     icon_cache->origin = USING_NO_ICON;
     icon_cache->prev_pixmap = None;
@@ -2039,18 +2047,12 @@ default_icon_at_size (int width, int height)
 
     GdkPixbuf *base;
     GdkPixbuf *nbase;
-    XfceIconTheme *theme;
     base = NULL;
     
     if (width > 0) {
-        theme = xfce_icon_theme_get_for_screen (NULL);
-        if (theme) {
-            if (!base) base = xfce_icon_theme_load_category (theme, XFCE_ICON_CATEGORY_UNKNOWN, width);
-            if (!base) base = xfce_icon_theme_load (theme, "xfce-unknown", width);
-            if (!base) base = xfce_icon_theme_load (theme, "unknown", width);
-        
-            g_object_unref (G_OBJECT (theme));
-        }
+        if (!base) base = xfce_themed_icon_load_category (XFCE_ICON_CATEGORY_UNKNOWN, width);
+        if (!base) base = xfce_themed_icon_load ("xfce-unknown", width);
+        if (!base) base = xfce_themed_icon_load ("unknown", width);
         if (base) {
         	nbase = gdk_pixbuf_copy (base);
         	g_object_unref (G_OBJECT (base));
@@ -2252,11 +2254,12 @@ p_netk_set_desktop_layout (Screen * xscreen, int rows, int columns)
     data[1] = columns;
     data[2] = rows;
     data[3] = _NET_WM_TOPLEFT;
+
     p_netk_error_trap_push ();
 
     XChangeProperty (gdk_display, RootWindowOfScreen (xscreen),
                      p_netk_atom_get ("_NET_DESKTOP_LAYOUT"), XA_CARDINAL, 32,
-                     PropModeReplace, (guchar *) & data, sizeof (data) / 4);
+                     PropModeReplace, (guchar *) &data, 4);
 
     p_netk_error_trap_pop ();
 }
