@@ -49,7 +49,7 @@
 
 #define DIGITS_WIDTH 234
 #define DIGITS_HEIGHT 74
-static unsigned char digits_bits[] = {
+static const unsigned char digits_bits[] = {
     0x0e, 0xe0, 0x38, 0x80, 0xe3, 0x38, 0x8e, 0xe3, 0x38, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x04, 0x41, 0x51, 0x10, 0x40,
@@ -297,7 +297,8 @@ xfce_clock_get_type (void)
 	    NULL,		/* class_data */
 	    sizeof (XfceClock),
 	    0,			/* n_preallocs */
-	    (GInstanceInitFunc) xfce_clock_init
+	    (GInstanceInitFunc) xfce_clock_init,
+	    NULL		/* value_table */
 	};
 
 	clock_type =
@@ -367,7 +368,7 @@ xfce_clock_realize (GtkWidget * widget)
     if (!(clock->digits_bmap))
     {
 	clock->digits_bmap =
-	    gdk_bitmap_create_from_data (widget->window, digits_bits,
+	    gdk_bitmap_create_from_data (widget->window, (gchar*) digits_bits,
 					 DIGITS_WIDTH, DIGITS_HEIGHT);
     }
     if (!(clock->timer))
